/*:
* @plugindesc （v1.07）双MP-轩辕剑体力数据
* @author 小c
* @version 1.05
* @date 1/4/2020
*
* @param Default Max TP Value
* @text 默认TP上限
* @type number
* @min 0
* @desc 当上限公式执行失败时取此值
* @default 100
*
* @param Default Max TP Formula
* @text 默认TP上限公式
* @type note
* @desc 默认TP公式，user为当前角色，对没有任何标签的目标执行此公式
* @default "//变量user为当前角色或敌人，tp为TP上限取值\nif(user.isActor())\n    tp = user.level * 3;\nelse\n    tp = 100;"
*
* @param Damage Image Lines
* @text 伤害数字总行数
* @type number
* @min 1
* @desc 伤害图总行数
* @default 5
*
* @param TP Popup Base Line
* @text TP伤害基准行
* @type number
* @min 0
* @default 0
*
* @param Gain TP From Damage
* @text 受伤时获得TP
* @type boolean
* @on 获取
* @off 不获取
* @default false
*
* @param TP Recover Variance
* @text TP伤害默认散度
* @type number
* @min 0
* @max 100
* @default 20
* @desc TP技能的默认散度，当没有配置TP散度且技能也没有伤害时取此值。
*
* @param TP Recover Critical
* @text TP效果默认暴击
* @type boolean
* @on 暴击
* @off 不暴击
* @desc TP技能默认是否暴击，当没有配置TP暴击且技能也没有伤害时取此值。
* @default true
*
* @help
* 轩辕剑命灵体系统，将TP转换为MP体系。
* 注意：将该插件放在YEP战斗核心以及暴击控制的下面。
* 
* 插件参数说明
* TP伤害基准行
* 参考img/system/damage.png，图片从上到下分多行，顶端称为第0行。
* 默认为
* 0行：HP伤害
* 1行：HP恢复
* 2行：MP伤害
* 3行：MP恢复
* 4行：miss
* 可在miss下方扩展数值用于TP伤害与恢复。
* 若此参数为x，则x行用于TP伤害，(x+1)行用于TP恢复。
* 在设置此参数的同时，务必将伤害数字总行数同时指定。
*
* 插件主要用途
* 1. TP上限
* 角色、职业、敌人标签
* <TP Value: x>
* 指定TP上限为固定值x
*
* <TP Formula Eval>
* // JavaScript code
* tp = user.level * 10;
* </TP Formula Eval>
* TP上限公式 user表示当前角色，tp的取值即为上限值。
*
* 2. TP恢复或TP伤害
* 技能、物品标签
* <TP Recover: x>
* 执行恢复TP x点。
* <TP Damage: x>
* 执行伤害TP x点。
* <TP Variance: x> TP技能物品的分散度为x，取值[0, 100]
* 若没有标签，且技能伤害中有HP或MP效果，TP分散度与伤害分散度一致。
* 若没有标签，且技能伤害类型为无，分散度以插件参数为准。
*
* <TP Recover Eval>
* // JavaScript code
* value = user.mat / 10 + target.maxTp() / 20;
* variance = 25;
* </TP Recover Eval>
* TP恢复公式，user为技能使用者，target为技能目标。
* variance为效果分散度，可忽略。
*
* <TP Damage Eval>
* // JavaScript code
* value = user.atk - target.def;
* variance = 30;
* </TP Damage Eval>
* TP伤害公式，变量说明与恢复相同。
*
* TP伤害暴击
* <TP Critical> TP伤害可暴击
* <TP Not Critical> TP伤害不可暴击
* 若无此标签，以技能伤害设置为准，技能无伤害则以插件参数为准。
*
* 3. TP上限【由人物特性决定】
* 角色、职业、武器、防具、敌人、状态标签
* <TP Rate: x> TP上限比率为x
* 当x=1.1时相当于MV特性中的【最大HP * 110%】。
*
* <TP Rate Eval>
* // JavaScript code
* rate = user.level / 10 + 1;
* </TP Rate Eval>
* 用公式计算比率，user为当前角色。
*
* <TP Plus: x> TP上限增加x。
* x=20相当于装备基础属性中的【最大HP: 20】。
*
* <TP Plus Eval>
* // JavaScript code
* plus = user.level + 1;
* </TP Plus Eval>
* 用公式计算增加参数，user为当前角色。
*
* 4. TP永久提升【上限药品、事件】
* 物品标签
* <Add TP Param: x>
* 使用该物品永久增加TP上限x。
*
* 插件指令
* ChangeMaxTp actorId value
* 改变指定角色的TP上限
* 例：ChangeMaxTp 1 +100
* 增加1号角色TP上限100点。
*
* <Add TP Param Eval>
* // JavaScript code
* plus = user.atk;
* </Add TP Param Eval>
* 用公式计算增加参数，user为当前角色。
*
* 5. TP技能消耗比率
* 技能消耗TP比率 比如【TP消耗-10%】等。
* 角色、职业、武器、防具、敌人、状态标签
* <TP Cost Rate: x>
* TP消耗比率为x，x=0.9表示消耗减少10%。
* 
* <TP Cost Rate Eval>
* // JavaScript code
* rate = 1 - user.level / 100;
* </TP Cost Rate Eval>
* 用公式计算比率，user为当前角色。
*
* 注：预留JavaScript函数
* battler.maxTp() 返回战斗这的tp上限
* battler.addTPParam(value) 增加战斗者tp上限value 
*
* action.isTpEffect() 行动是否为TP效果
* action.isTpDamage() 行动是否为TP伤害
* action.isTpRecover() 行动是否为TP恢复
*/

var Imported = Imported || {};
Imported.CP_TPCore = true;

var TPManager = TPManager || {};
var params = PluginManager.parameters("CP_TPCore");

TPManager.DEFAULT_MAX_TP_VALUE = Number(params["Default Max TP Value"]) || 0;
TPManager.DEFAULT_MAX_TP_FORMULA = JSON.parse(params["Default Max TP Formula"]).trim();

TPManager.TP_POPUP_BASE_LINE = Number(params["TP Popup Base Line"]) || 0;
TPManager.DAMAGE_IMAGE_LINES = Number(params["Damage Image Lines"]) || 0;

TPManager.GAIN_TP_FROM_DAMAGE = eval(params["Gain TP From Damage"]);

TPManager.TP_RECOVER_VARIANCE = Number(params["TP Recover Variance"]) || 0;
TPManager.TP_RECOVER_CRITICAL = eval(params["TP Recover Critical"]) || false;

TPManager.DATABASE_LOADED = DataManager.isDatabaseLoaded;
DataManager.isDatabaseLoaded = function(){
	if(!TPManager.DATABASE_LOADED.call(this))
		return false;

	TPManager.loadTPFormula($dataActors);
	TPManager.loadTPFormula($dataClasses);
	TPManager.loadTPFormula($dataEnemies);

	TPManager.loadTPRecovery($dataSkills);
	TPManager.loadTPRecovery($dataItems);

	TPManager.loadTPRateAndPlus($dataActors);
	TPManager.loadTPRateAndPlus($dataClasses);
	TPManager.loadTPRateAndPlus($dataWeapons);
	TPManager.loadTPRateAndPlus($dataArmors);
	TPManager.loadTPRateAndPlus($dataEnemies);
	TPManager.loadTPRateAndPlus($dataStates);

	TPManager.loadTPAddItem();

	TPManager.loadTPCostRate($dataActors);
	TPManager.loadTPCostRate($dataClasses);
	TPManager.loadTPCostRate($dataWeapons);
	TPManager.loadTPCostRate($dataArmors);
	TPManager.loadTPCostRate($dataEnemies);
	TPManager.loadTPCostRate($dataStates);

	return true;
};

TPManager.loadTPFormula = function(array){
	for(var i = 1; i < array.length; i++){
		var item = array[i];

		item.tpFormula = null;
		item.tpValue = 0;

		if(item.meta["TP Value"]){
			var value = Number(item.meta["TP Value"]) || 0;
			item.tpValue = value;
		}

		if(item.meta["TP Formula Eval"]){
			var reg = /<TP Formula Eval>([\s\S]*)<\/TP Formula Eval>/;
			reg.exec(item.note);
			item.tpFormula = RegExp.$1;
		}
	}
};

TPManager.loadTPRecovery = function(array){
	for(var i = 1; i < array.length; i++){
		var item = array[i];

		item.tpRecover = {};
		item.tpRecover.value = 0;
		item.tpRecover.type = 0;
		item.tpRecover.eval = null;
		item.tpRecover.critical = item.damage.critical;
		item.tpRecover.variance = item.damage.variance;
		if(item.damage.type === 0){
			item.tpRecover.variance = this.TP_RECOVER_VARIANCE;
			item.tpRecover.critical = this.TP_RECOVER_CRITICAL;
		}

		if(item.meta["TP Recover"]){
			var tpr = Number(item.meta["TP Recover"]) || 0;
			item.tpRecover.value = tpr;
			item.tpRecover.type = 1;
		}

		if(item.meta["TP Recover Eval"]){
			var reg = /<TP Recover Eval>([\s\S]*)<\/TP Recover Eval>/;
			reg.exec(item.note);
			item.tpRecover.eval = RegExp.$1;
			item.tpRecover.type = 1;
		}

		if(item.meta["TP Damage"]){
			var tpd = Number(item.meta["TP Damage"]) || 0;
			item.tpRecover.value = tpd;
			item.tpRecover.type = -1;
		}

		if(item.meta["TP Damage Eval"]){
			var reg = /<TP Damage Eval>([\s\S]*)<\/TP Damage Eval>/;
			reg.exec(item.note);
			item.tpRecover.eval = RegExp.$1;
			item.tpRecover.type = -1;
		}

		if(item.meta["TP Variance"]){
			var tpv = Number(item.meta["TP Variance"]) || 0;
			item.tpRecover.variance = tpv;
		}

		if(item.meta["TP Critical"])
			item.tpRecover.critical = true;
		if(item.meta["TP Not Critical"])
			item.tpRecover.critical = false;
	}
};

TPManager.loadTPRateAndPlus = function(array){
	for(var i = 1; i < array.length; i++){
		var item = array[i];
		item.tpRate = 1;
		item.tpRateEval = null;
		item.tpPlus = 0;
		item.tpPlusEval = null;

		if(item.meta["TP Rate"]){
			var rate = Number(item.meta["TP Rate"]) || 1;
			item.tpRate = rate;
		}

		if(item.meta["TP Rate Eval"]){
			var reg = /<TP Rate Eval>([\s\S]*)<\/TP Rate Eval>/;
			reg.exec(item.note);
			item.tpRateEval = RegExp.$1;
		}

		if(item.meta["TP Plus"]){
			var plus = Number(item.meta["TP Plus"]) || 0;
			item.tpPlus = plus;
		}

		if(item.meta["TP Plus Eval"]){
			var reg = /<TP Plus Eval>([\s\S]*)<\/TP Plus Eval>/;
			reg.exec(item.note);
			item.tpPlusEval = RegExp.$1;
		}
	}
};

TPManager.loadTPAddItem = function(){
	for(var i = 1; i < $dataItems.length; i++){
		var item = $dataItems[i];
		item.tpParamPlus = 0;
		item.tpParamPlusEval = null;

		if(item.meta["Add TP Param"]){
			var add = Number(item.meta["Add TP Param"]) || 0;
			item.tpParamPlus = add;
		}

		if(item.meta["Add TP Param Eval"]){
			var reg = /<Add TP Param Eval>([\s\S]*)<\/Add TP Param Eval>/;
			reg.exec(item.note);
			item.tpParamPlusEval = RegExp.$1;
		}
	}
};

TPManager.loadTPCostRate = function(array){
	for(var i = 1; i < array.length; i++){
		var item = array[i];
		item.tpCostRate = 1;
		item.tpCostRateEval = null;

		if(item.meta["TP Cost Rate"]){
			var rate = Number(item.meta["TP Cost Rate"]) || 0;
			item.tpCostRate = rate;
		}

		if(item.meta["TP Cost Rate Eval"]){
			var reg = /<TP Cost Rate Eval>([\s\S]*)<\/TP Cost Rate Eval>/;
			reg.exec(item.note);
			item.tpCostRateEval = RegExp.$1;
		}
	}
};

TPManager.INIT_MEMBERS = Game_BattlerBase.prototype.initMembers;
Game_BattlerBase.prototype.initMembers = function() {
   TPManager.INIT_MEMBERS.call(this);

   this._tpPlus = 0;//体力上限永久提升值
};

//额外体力值
Game_Battler.prototype.tpRateFromItem = function(item){
	var rate = 1;
	if(!item) return rate;

	rate = item.tpRate;

	if(item.tpRateEval){
		var user = this;

		try{
			eval(item.tpRateEval);
		}catch(err){
			rate = 1;
			console.error(err);
		}
	}

	return rate;
};

Game_Battler.prototype.tpPlusFromItem = function(item){
	var plus = 0;
	if(!item) return plus;

	plus = item.tpPlus;

	if(item.tpPlusEval){
		var user = this;

		try{
			eval(item.tpPlusEval);
		}catch(err){
			plus = 0;
			console.error(err);
		}
	}

	return plus;
};

Game_Battler.prototype.tpRateFromStates = function(){
	var rate = 1;
	var states = this.states();
	for(var i = 0; i < states.length; i++){
		rate *= this.tpRateFromItem(states[i]);
	}

	return rate;
};

Game_Battler.prototype.tpPlusFromStates = function(){
	var plus = 0;
	var states = this.states();
	for(var i = 0; i < states.length; i++){
		plus += this.tpPlusFromItem(states[i]);
	}

	return plus;
};

Game_Actor.prototype.tpRateFromEquips = function(){
	var rate = 1;
	var equips = this.equips();
	for(var i = 0; i < equips.length; i++){
		rate *= this.tpRateFromItem(equips[i]);
	}

	return rate;
};

Game_Actor.prototype.tpPlusFromEquips = function(){
	var plus = 0;
	var equips = this.equips();
	for(var i = 0; i < equips.length; i++){
		plus += this.tpPlusFromItem(equips[i]);
	}

	return plus;
};

Game_Actor.prototype.baseTPRate = function(){
	return this.tpRateFromItem(this.actor()) * this.tpRateFromItem(this.currentClass())
		 * this.tpRateFromEquips() * this.tpRateFromStates();
};

Game_Actor.prototype.baseTPPlus = function(){
	return this._tpPlus + this.tpPlusFromItem(this.actor()) + this.tpPlusFromItem(this.currentClass()) 
		+ this.tpPlusFromEquips() + this.tpPlusFromStates();
};

Game_Enemy.prototype.baseTPRate = function(){
	return this.tpRateFromItem(this.enemy()) * this.tpRateFromStates();
};

Game_Enemy.prototype.baseTPPlus = function(){
	return this.tpPlusFromItem(this.enemy()) + this.tpPlusFromStates();
};

Game_Battler.prototype.initTp = function() {};

TPManager.CHANGE_TP_BY_DAMAGE = Game_Battler.prototype.chargeTpByDamage;
Game_Battler.prototype.chargeTpByDamage = function(damageRate) {
	if(TPManager.GAIN_TP_FROM_DAMAGE)
		TPManager.CHANGE_TP_BY_DAMAGE.call(this, damageRate);
};

Game_BattlerBase.prototype.isPreserveTp = function() {
    return true;
};

Game_Actor.prototype.maxTpBase = function() {
    var tp = TPManager.DEFAULT_MAX_TP_VALUE;
    var ctp = this.currentClass().tpValue;
    var atp = this.actor().tpValue;

    var formula = this.currentClass().tpFormula || this.actor().tpFormula || TPManager.DEFAULT_MAX_TP_FORMULA;
    if(formula == TPManager.DEFAULT_MAX_TP_FORMULA){
    	if(ctp > 0)
    		return ctp;
    	else if(atp > 0)
    		return atp;
    }

	var user = this;
	
	try{
    	eval(formula);
    }catch(err){
    	tp = TPManager.DEFAULT_MAX_TP_VALUE;
    	console.error(err);
    }

    return tp;
};	

Game_Enemy.prototype.maxTpBase = function() {
    var tp = TPManager.DEFAULT_MAX_TP_VALUE;
    var etp = this.enemy().tpValue;

    var formula = this.enemy().tpFormula || TPManager.DEFAULT_MAX_TP_FORMULA;
   
    if(formula == TPManager.DEFAULT_MAX_TP_FORMULA){
    	if(etp > 0)
    		return etp;
    }

    var user = this;

	try{
    	eval(formula);
    }catch(err){
    	tp = TPManager.DEFAULT_MAX_TP_VALUE;
    	console.error(err);
    }
     
    return tp;
};

Game_Battler.prototype.maxTp = function(){
	return Math.round((this.maxTpBase() + this.baseTPPlus()) * this.baseTPRate());
};

Game_Battler.prototype.regenerateTp = function(){
    var value = Math.floor(this.maxTp() * this.trg);
    this.gainTp(value);
};

TPManager.RECOVER_ALL = Game_BattlerBase.prototype.recoverAll;
Game_BattlerBase.prototype.recoverAll = function() {
    TPManager.RECOVER_ALL.call(this);
    this._tp = this.maxTp();
};

Game_Actor.prototype.addTPParam = function(value){
	this._tpPlus += value;
};

TPManager.INIT_ACTION = Game_Action.prototype.initialize;
Game_Action.prototype.initialize = function(subject, forcing){
	TPManager.INIT_ACTION.call(this, subject, forcing);

	this._tpCriticalTest = false; //临时变量，判定是否计算tp暴击
};

Game_Action.prototype.isTpEffect = function(){
	var item = this.item();

	return item.tpRecover.value || item.tpRecover.eval;
};

Game_Action.prototype.isTpDamage = function(){
	var item = this.item();

	return this.isTpEffect() && item.tpRecover.type === -1;
};

Game_Action.prototype.isTpRecover = function(){
	var item = this.item();

	return this.isTpEffect() && item.tpRecover.type === 1;
};

TPManager.APPLY_ACTION = Game_Action.prototype.apply;
Game_Action.prototype.apply = function(target) {
	TPManager.APPLY_ACTION.call(this, target);

    var item = this.item();
    if(item.tpParamPlus > 0 || item.tpParamPlusEval)
    	this.addTPParam(target);
};

//在命中的最后 执行TP操作
TPManager.APPLY_ACTION_USER_EFFECT = Game_Action.prototype.applyItemUserEffect;
Game_Action.prototype.applyItemUserEffect = function(target) {
    TPManager.APPLY_ACTION_USER_EFFECT.call(this, target);

    if(this.isTpEffect())
    	this.executeTpRecovery(target);
};

Game_Action.prototype.executeTpRecovery = function(target){
	 var result = target.result();
	 var item = this.item();
	 var variance = item.tpRecover.variance;
	 var critical = false;

	 var value = 0;
	 var user = this.subject();

	 try{
	 	if(item.tpRecover.eval)
	 		eval(item.tpRecover.eval);
	 	else
	 		value = item.tpRecover.value;
	 }catch(err){
	 	value = item.tpRecover.value;
	 	console.error(err);
	 }

	 if(value < 0)
	 	value = 0;

	 variance = variance.clamp(0, 100);
	 value = this.applyVariance(value, variance);

	 if(item.tpRecover.critical && Math.random() < this.tpCri(target)){
	 	value = this.applyCritical(value);
	 	target.result().tpCritical = true;
	 }

	 this.makeSuccess(target);
	 target.gainTp(Math.round(value * item.tpRecover.type));
};

//TP暴击部分
TPManager.CLEAR_ACTION_RESULT = Game_ActionResult.prototype.clear;
Game_ActionResult.prototype.clear = function(){
	TPManager.CLEAR_ACTION_RESULT.call(this);

	this.tpCritical = false; //独立出TP暴击
};

TPManager.ACTION_ITEM_CRI = Game_Action.prototype.itemCri;
Game_Action.prototype.itemCri = function(target){
	if(!this._tpCriticalTest)
		return TPManager.ACTION_ITEM_CRI.call(this, target);

	var item = this.item();
	var critical = item.damage.critical;
	item.damage.critical = item.tpRecover.critical;

	var rate = TPManager.ACTION_ITEM_CRI.call(this, target);
	item.damage.critical = critical;

	return rate;
};

Game_Action.prototype.tpCri = function(target){
	this._tpCriticalTest = true;
	var rate = this.itemCri(target);
	this._tpCriticalTest = false;

	return rate;
};

//上限部分
Game_Action.prototype.addTPParam = function(target){
	var item = this.item();
	var plus = 0;

	if(item.tpParamPlus > 0)
		plus = item.tpParamPlus;
	else if(item.tpParamPlusEval){
		var user = this.subject();
		try{
			eval(item.tpParamPlusEval);
		}catch(err){
			plus = 0;
			console.error(err);
		}
	}

	target.addTPParam(plus);
};

//体力上限物品允许使用
TPManager.HAS_ITEM_ANY_VALID_EFFECT = Game_Action.prototype.hasItemAnyValidEffects;
Game_Action.prototype.hasItemAnyValidEffects = function(target) {
	var result = TPManager.HAS_ITEM_ANY_VALID_EFFECT.call(this, target);
	var item = this.item();
	if(item.tpParamPlus > 0 || item.tpParamPlusEval)
		result = true;
    return result;
};

//TP消耗部分
Game_Battler.prototype.tpCostRateFromItem = function(item){
	var rate = 1;
	if(!item) return rate;

	rate = item.tpCostRate;

	if(item.tpCostRateEval){
		var user = this;

		try{
			eval(item.tpCostRateEval);
		}catch(err){
			rate = 1;
			console.error(err);
		}
	}

	return rate;
};

Game_Battler.prototype.tpCostRateFromStates = function(){
	var rate = 1;
	var states = this.states();
	for(var i = 0; i < states.length; i++){
		rate *= this.tpCostRateFromItem(states[i]);
	}

	return rate;
};

Game_Actor.prototype.tpCostRateFromEquips = function(){
	var rate = 1;
	var equips = this.equips();
	for(var i = 0; i < equips.length; i++){
		rate *= this.tpCostRateFromItem(equips[i]);
	}

	return rate;
};

Game_Actor.prototype.tpCostRate = function(){
	return this.tpCostRateFromItem(this.actor()) * this.tpCostRateFromItem(this.currentClass()) * this.tpCostRateFromEquips() 
		* this.tpCostRateFromStates();
};


Game_Enemy.prototype.tpCostRate = function(){
	return this.tpCostRateFromItem(this.enemy()) * this.tpCostRateFromStates();
};

Game_Battler.prototype.skillTpCost = function(skill) {
    var cost = Math.round(Game_BattlerBase.prototype.skillTpCost.call(this, skill) * this.tpCostRate());

    return cost > 0 ? cost : 0;
};

//伤害弹出部分
//构造数字
TPManager.SETUP_DAMAGE_SPRITE = Sprite_Damage.prototype.setup;
Sprite_Damage.prototype.setup = function(target) {
	TPManager.SETUP_DAMAGE_SPRITE.call(this, target);

	var result = target.result();
	if(Imported.YEP_BattleEngineCore)
		result = this._result;

   if(!result.missed && !result.evaded && target.isAlive() && result.tpDamage !== 0)
        this.createDigits(TPManager.TP_POPUP_BASE_LINE, result.tpDamage);

    if (result.tpDamage !== 0 && result.tpCritical)
        this.setupCriticalEffect();
};

Sprite_Damage.prototype.digitHeight = function() {
    return this._damageBitmap ? this._damageBitmap.height / TPManager.DAMAGE_IMAGE_LINES : 0;
};

TPManager.BATTLER_START_DAMAGE_POPUP = Game_Battler.prototype.startDamagePopup;
Game_Battler.prototype.startDamagePopup = function() {
	if(Imported.YEP_BattleEngineCore)
		this.startDamagePopupForYEPBattle();
	else
		TPManager.BATTLER_START_DAMAGE_POPUP.call(this);
};

Game_Battler.prototype.startDamagePopupForYEPBattle = function() {
	var result = this.result();
    if(result.missed || result.evaded) {
	    var copyResult = JsonEx.makeDeepCopy(result);
	    copyResult.hpAffected = false;
	    copyResult.mpDamage = 0;
	    copyResult.tpDamage = 0;
	    this._damagePopup.push(copyResult);
    }
    if (result.hpAffected) {
	    var copyResult = JsonEx.makeDeepCopy(result);
	    copyResult.mpDamage = 0;
	    copyResult.tpDamage = 0;
	    this._damagePopup.push(copyResult);
    }
    if (result.mpDamage !== 0) {
	    var copyResult = JsonEx.makeDeepCopy(result);
	    copyResult.hpAffected = false;
	    copyResult.tpDamage = 0;
	    this._damagePopup.push(copyResult);
    }
    if(result.tpDamage != 0){
 	    var copyResult = JsonEx.makeDeepCopy(result);
	    copyResult.hpAffected = false;
	    copyResult.mpDamage = 0;
	    this._damagePopup.push(copyResult);
	}
};

TPManager.PLUGIN_COMMAND = Game_Interpreter.prototype.pluginCommand;
Game_Interpreter.prototype.pluginCommand = function(command, args) {
	TPManager.PLUGIN_COMMAND.call(this, command, args);

	if(command == 'ChangeMaxTp'){
		var actorId = Number(args[0]) || 0;
		var value = Number(args[1]) || 0;
		
		var actor = $gameActors.actor(actorId);
		if(actor)
			actor.addTPParam(value);
	}
};